/*---------------------------------------------------------------------------------
	$Id: template.c,v 1.2 2005/09/07 20:06:06 wntrmute Exp $

	Simple ARM7 stub (sends RTC, TSC, and X/Y data to the ARM 9)

	$Log: template.c,v $
	Revision 1.2  2005/09/07 20:06:06  wntrmute
	updated for latest libnds changes
	
	Revision 1.8  2005/08/03 05:13:16  wntrmute
	corrected sound code


---------------------------------------------------------------------------------*/
#include <nds.h>

#include <nds/bios.h>
#include <nds/arm7/touch.h>
#include <nds/arm7/clock.h>

#include <dswifi7.h>
#include "../../ipcex.h"
#include "linkreset_arm7.h"

extern	void ret_menu7_R4(void);	//====== R4TF was added. by Rudolph (2007/05/23)

//---------------------------------------------------------------------------------
void VblankHandler(void) {
	//Wifi_Update(); // update wireless in vblank
	//VBLANK_INTR_WAIT_FLAGS |= IRQ_VBLANK;
	//REG_IF |= IRQ_VBLANK;
}

// callback to allow wifi library to notify arm9
void arm7_synctoarm9() { // send fifo message
	REG_IPC_FIFO_TX = 0x87654321;
}

// interrupt handler to allow incoming notifications from arm9
void arm7_fifo() { // check incoming fifo messages
	u32 msg = REG_IPC_FIFO_RX;
	if ( msg == 0x87654321 ) {
		Wifi_Sync();
	}
}

void Timer_50ms(void) {
	Wifi_Update();
}

__attribute__((noinline)) static void main_Proc_Reset(ERESET RESET)
{
	ret_menu7_R4();
	*(vu16*)(0x04000208) = 0;       //REG_IME = IME_DISABLE;
	*((vu32*)0x027FFE34) = *((vu32*)0x027FFFF8);
	asm("swi 0x00");                //swiSoftReset();
	asm("bx lr");
}
//---------------------------------------------------------------------------------
int main(int argc, char ** argv) {
//---------------------------------------------------------------------------------
	//LOADNDS->PATH = 0;
	REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR; // enable & prepare fifo asap
	// Reset the clock if needed
	//rtcReset();

	//enable sound
	//powerON(POWER_SOUND);
	IPC->soundData = 0;

	irqInit();
	irqSet(IRQ_VBLANK, VblankHandler);
	irqEnable(IRQ_VBLANK);
	TIMER2_DATA = -1638; // 6553.1 * 256 cycles = ~50ms;
	TIMER2_CR = 0x00C2; // enable, irq, 1/256 clock
	irqSet(IRQ_TIMER2, Timer_50ms); // Timer IRQ ̃ZbgAbv
	irqEnable(IRQ_TIMER2);
	
	irqSet(IRQ_WIFI, Wifi_Interrupt); // set up wifi interrupt
	irqEnable(IRQ_WIFI);

	{ // sync with arm9 and init wifi
		u32 fifo_temp;   

		while(1) { // wait for magic number
			while ( REG_IPC_FIFO_CR & IPC_FIFO_RECV_EMPTY ) swiWaitForVBlank();
			fifo_temp = REG_IPC_FIFO_RX;
			if ( fifo_temp == 0x12345678 ) break;
		}
		while ( REG_IPC_FIFO_CR & IPC_FIFO_RECV_EMPTY ) swiWaitForVBlank();
		fifo_temp = REG_IPC_FIFO_RX; // give next value to wifi_init
		Wifi_Init(fifo_temp);
   	
		irqSet(IRQ_FIFO_NOT_EMPTY, arm7_fifo); // set up fifo irq
		irqEnable(IRQ_FIFO_NOT_EMPTY);
		REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_RECV_IRQ;

		Wifi_SetSyncHandler(arm7_synctoarm9); // allow wifi lib to notify arm9
	} // arm7 wifi init complete

	IPCEX->RESET=RESET_NULL;
	// Keep the ARM7 out of main RAM
	for ( ;; ) {
		swiWaitForVBlank();
		if(IPCEX->RESET!=RESET_NULL){
			main_Proc_Reset(IPCEX->RESET);
			while(1);
		}
	}
}


